/*
 * Class to recursively draw broccoli
 * 
 * From Williams College, CSC 134
 * 
 * Updated by Jim Teresco for CSC 252, The College of Saint Rose, Fall 2013
 * Code cleanup, Spring 2016
 * 
 * $Id: BroccoliBranch.java 2924 2016-03-31 00:40:33Z terescoj $ 
 */
import objectdraw.*;
import java.awt.*;

public class BroccoliBranch {

    // How much broccoli shrinks each call
    private static final double TOP_FRACT = 0.8;
    // How little a branch must be before broccoli stops expanding
    private static final double GROWTH_LIMIT = 20.0;
    // The size (width and height) of a flower
    private static final int FLOWER_SIZE = 3;

    // branches of broccoli: our recursive data structure
    // note that in this case this not one "next", but three!
    // these will be null if we are at the base case
    private BroccoliBranch left, center, right;

    // stem of broccoli-- every BroccoliBranch has one
    private AngLine stem;

    // Flower of broccoli plant, null except in base case (and that's how we'll tell)
    private FilledOval flower;

    /**
     * Draw broccoli by recursively drawing branches (and eventually flower)
     */
    public BroccoliBranch(Location startLocation, double size, double direction, DrawingCanvas canvas) {

        // Draw stem and color green (we always have this)
        stem = new AngLine(startLocation,size,direction,canvas);
        stem.setColor(Color.green);

        Location destLocation = stem.getDest(); // end of stem

        if (size > GROWTH_LIMIT) {  // we're big enough to keep growing
            left = new BroccoliBranch(destLocation, size * TOP_FRACT,
                direction + Math.PI/9.0, canvas);
            center = new BroccoliBranch(destLocation, size * TOP_FRACT,
                direction, canvas);
            right = new BroccoliBranch(destLocation, size * TOP_FRACT,
                direction - Math.PI/9.0, canvas);
        } 
        else {  
            // draw flower when small enough -- note this will be null except
            // in the base case, and will be used to determine the base for
            // recursive methods
            flower = new FilledOval(destLocation,FLOWER_SIZE,FLOWER_SIZE,canvas);
            flower.setColor(Color.yellow);
        }
    }

    // standard move method, but this one's recursive
    public void move(double x, double y) {

        stem.move(x,y);     // move stem in all cases

        if (flower == null)  {
            // not the base case, so move rest of broccoli - three branches
            left.move(x,y);
            center.move(x,y);
            right.move(x,y);
        } 
        else {
            // move flower only, it's the base case
            flower.move(x,y);
        }
    }
    
    // contains, also needs recursion here
    public boolean contains(Location point) {
    
        // first, check stem, since we always know we have one
        if (stem.contains(point)) return true;
        
        // ok, stem didn't contain the point, so we keep checking
        // do base case next: if flower is not null, it's the only hope we
        // have to contain the point, so it would be our answer
        if (flower != null) {
            return flower.contains(point);
        }
        
        // we must be in a recursive case, so we need to try all three branches
        return left.contains(point) || center.contains(point) || right.contains(point);
        
    }
    
}